@@ -23,11 +23,12 @@
 #include <config.h>
 #endif
 
+#include "gedit-replace-dialog.h"
+
 #include <string.h>
 #include <glib/gi18n.h>
 #include <gdk/gdkkeysyms.h>
 
-#include "gedit-replace-dialog.h"
 #include "gedit-history-entry.h"
 #include "gedit-document.h"
 
@@ -47,6 +48,7 @@ struct _GeditReplaceDialogPrivate
 	GtkWidget *regex_checkbutton;
 	GtkWidget *backwards_checkbutton;
 	GtkWidget *wrap_around_checkbutton;
+	GtkWidget *close_button;
 
 	GeditDocument *active_document;
 
@@ -538,6 +540,7 @@ gedit_replace_dialog_class_init (GeditReplaceDialogClass *klass)
 	gtk_widget_class_bind_template_child_private (widget_class, GeditReplaceDialog, regex_checkbutton);
 	gtk_widget_class_bind_template_child_private (widget_class, GeditReplaceDialog, backwards_checkbutton);
 	gtk_widget_class_bind_template_child_private (widget_class, GeditReplaceDialog, wrap_around_checkbutton);
+	gtk_widget_class_bind_template_child_private (widget_class, GeditReplaceDialog, close_button);
 }
 
 static void
@@ -680,6 +683,7 @@ gedit_replace_dialog_init (GeditReplaceDialog *dlg)
 	dlg->priv->replace_entry = gedit_history_entry_new ("replace-with-entry", TRUE);
 	gtk_widget_set_hexpand (GTK_WIDGET (dlg->priv->replace_entry), TRUE);
 	dlg->priv->replace_text_entry = gedit_history_entry_get_entry (GEDIT_HISTORY_ENTRY (dlg->priv->replace_entry));
+	gtk_entry_set_placeholder_text (GTK_ENTRY (dlg->priv->replace_text_entry), _("Nothing"));
 	gtk_entry_set_activates_default (GTK_ENTRY (dlg->priv->replace_text_entry), TRUE);
 	gtk_grid_attach_next_to (GTK_GRID (dlg->priv->grid),
 				 dlg->priv->replace_entry,
@@ -744,14 +748,41 @@ GtkWidget *
 gedit_replace_dialog_new (GeditWindow *window)
 {
 	GeditReplaceDialog *dialog;
+	gboolean use_header;
 
 	g_return_val_if_fail (GEDIT_IS_WINDOW (window), NULL);
 
 	dialog = g_object_new (GEDIT_TYPE_REPLACE_DIALOG,
 			       "transient-for", window,
 			       "destroy-with-parent", TRUE,
+			       "use-header-bar", FALSE,
 			       NULL);
 
+	/* We want the Find/Replace/ReplaceAll buttons at the bottom,
+	 * so we turn off the automatic header bar, but we check the
+	 * setting and if a header bar should be used, we create it
+	 * manually and use it for the close button.
+	 */
+	g_object_get (gtk_settings_get_default (),
+	              "gtk-dialogs-use-header", &use_header,
+	              NULL);
+
+	if (use_header)
+	{
+		GtkWidget *header_bar;
+
+		header_bar = gtk_header_bar_new ();
+		gtk_header_bar_set_title (GTK_HEADER_BAR (header_bar), _("Find and Replace"));
+		gtk_header_bar_set_show_close_button (GTK_HEADER_BAR (header_bar), TRUE);
+		gtk_widget_show (header_bar);
+		gtk_window_set_titlebar (GTK_WINDOW (dialog), header_bar);
+	}
+	else
+	{
+		gtk_widget_set_no_show_all (dialog->priv->close_button, FALSE);
+		gtk_widget_show (dialog->priv->close_button);
+	}
+
 	return GTK_WIDGET (dialog);
 }
 
