@@ -51,9 +51,9 @@ struct _GeditTabPrivate
 
 	GtkWidget	       *info_bar;
 	GtkWidget	       *info_bar_hidden;
-	GtkWidget	       *print_preview;
 
 	GeditPrintJob          *print_job;
+	GtkWidget	       *print_preview;
 
 	GTask                  *task_saver;
 	GtkSourceFileSaverFlags save_flags;
@@ -183,17 +183,13 @@ remove_auto_save_timeout (GeditTab *tab)
 static void
 update_auto_save_timeout (GeditTab *tab)
 {
-	gboolean good_state;
 	GeditDocument *doc;
 
 	gedit_debug (DEBUG_TAB);
 
-	good_state = (tab->priv->state == GEDIT_TAB_STATE_NORMAL ||
-		      tab->priv->state == GEDIT_TAB_STATE_SHOWING_PRINT_PREVIEW);
-
 	doc = gedit_tab_get_document (tab);
 
-	if (good_state &&
+	if (tab->priv->state == GEDIT_TAB_STATE_NORMAL &&
 	    tab->priv->auto_save &&
 	    !gedit_document_is_untitled (doc) &&
 	    !gedit_document_get_readonly (doc))
@@ -278,14 +274,9 @@ gedit_tab_dispose (GObject *object)
 {
 	GeditTab *tab = GEDIT_TAB (object);
 
-	if (tab->priv->print_job != NULL)
-	{
-		g_object_unref (tab->priv->print_job);
-		tab->priv->print_job = NULL;
-		tab->priv->print_preview = NULL;
-	}
-
 	g_clear_object (&tab->priv->editor);
+	g_clear_object (&tab->priv->print_job);
+	g_clear_object (&tab->priv->print_preview);
 	g_clear_object (&tab->priv->task_saver);
 
 	clear_loading (tab);
@@ -486,7 +477,6 @@ set_view_properties_according_to_state (GeditTab      *tab,
 	view = gedit_tab_get_view (tab);
 
 	val = ((state == GEDIT_TAB_STATE_NORMAL) &&
-	       (tab->priv->print_preview == NULL) &&
 	       tab->priv->editable);
 	gtk_text_view_set_editable (GTK_TEXT_VIEW (view), val);
 
@@ -968,14 +958,7 @@ unrecoverable_saving_error_info_bar_response (GtkWidget *info_bar,
 {
 	GeditView *view;
 
-	if (tab->priv->print_preview != NULL)
-	{
-		gedit_tab_set_state (tab, GEDIT_TAB_STATE_SHOWING_PRINT_PREVIEW);
-	}
-	else
-	{
-		gedit_tab_set_state (tab, GEDIT_TAB_STATE_NORMAL);
-	}
+	gedit_tab_set_state (tab, GEDIT_TAB_STATE_NORMAL);
 
 	set_info_bar (tab, NULL, GTK_RESPONSE_NONE);
 
@@ -2154,6 +2137,23 @@ _gedit_tab_revert (GeditTab *tab)
 	load (tab, NULL, 0, 0);
 }
 
+static void
+close_printing (GeditTab *tab)
+{
+	if (tab->priv->print_preview != NULL)
+	{
+		gtk_widget_destroy (tab->priv->print_preview);
+	}
+
+	g_clear_object (&tab->priv->print_job);
+	g_clear_object (&tab->priv->print_preview);
+
+	/* destroy the info bar */
+	set_info_bar (tab, NULL, GTK_RESPONSE_NONE);
+
+	gedit_tab_set_state (tab, GEDIT_TAB_STATE_NORMAL);
+}
+
 static void
 saver_progress_cb (goffset   size,
 		   goffset   total_size,
@@ -2298,14 +2298,7 @@ save_cb (GtkSourceFileSaver *saver,
 	{
 		gedit_recent_add_document (doc);
 
-		if (tab->priv->print_preview != NULL)
-		{
-			gedit_tab_set_state (tab, GEDIT_TAB_STATE_SHOWING_PRINT_PREVIEW);
-		}
-		else
-		{
-			gedit_tab_set_state (tab, GEDIT_TAB_STATE_NORMAL);
-		}
+		gedit_tab_set_state (tab, GEDIT_TAB_STATE_NORMAL);
 
 		tab->priv->ask_if_externally_modified = TRUE;
 
@@ -2391,6 +2384,20 @@ _gedit_tab_save_async (GeditTab            *tab,
 		return;
 	}
 
+	/* The Save and Save As window actions are insensitive when the print
+	 * preview is shown, but it's still possible to save several documents
+	 * at once (with the Save All action or when quitting gedit). In that
+	 * case, the print preview is simply closed. Handling correctly the
+	 * document saving when the print preview is shown is more complicated
+	 * and error-prone, it doesn't worth the effort. (the print preview
+	 * would need to be updated when the filename changes, dealing with file
+	 * saving errors is also more complicated, etc).
+	 */
+	if (tab->priv->state == GEDIT_TAB_STATE_SHOWING_PRINT_PREVIEW)
+	{
+		close_printing (tab);
+	}
+
 	doc = gedit_tab_get_document (tab);
 	g_return_if_fail (!gedit_document_is_untitled (doc));
 
@@ -2465,8 +2472,7 @@ gedit_tab_auto_save (GeditTab *tab)
 		return G_SOURCE_CONTINUE;
 	}
 
-	if (tab->priv->state != GEDIT_TAB_STATE_NORMAL &&
-	    tab->priv->state != GEDIT_TAB_STATE_SHOWING_PRINT_PREVIEW)
+	if (tab->priv->state != GEDIT_TAB_STATE_NORMAL)
 	{
 		gedit_debug_message (DEBUG_TAB, "Retry after 30 seconds");
 
@@ -2540,6 +2546,12 @@ _gedit_tab_save_as_async (GeditTab                 *tab,
 		return;
 	}
 
+	/* See note at _gedit_tab_save_async(). */
+	if (tab->priv->state == GEDIT_TAB_STATE_SHOWING_PRINT_PREVIEW)
+	{
+		close_printing (tab);
+	}
+
 	tab->priv->task_saver = g_task_new (tab, cancellable, callback, user_data);
 
 	data = saver_data_new ();
@@ -2705,166 +2717,55 @@ done_printing_cb (GeditPrintJob       *job,
 			  tab->priv->state == GEDIT_TAB_STATE_SHOWING_PRINT_PREVIEW ||
 			  tab->priv->state == GEDIT_TAB_STATE_PRINTING);
 
-	if (tab->priv->state == GEDIT_TAB_STATE_SHOWING_PRINT_PREVIEW)
-	{
-		/* print preview has been destroyed... */
-		tab->priv->print_preview = NULL;
-	}
-	else
-	{
-		g_return_if_fail (GEDIT_IS_PROGRESS_INFO_BAR (tab->priv->info_bar));
-
-		set_info_bar (tab, NULL, GTK_RESPONSE_NONE); /* destroy the info bar */
-	}
-
 	/* TODO: check status and error */
 
-	if (result ==  GEDIT_PRINT_JOB_RESULT_OK)
+	if (result == GEDIT_PRINT_JOB_RESULT_OK)
 	{
 		store_print_settings (tab, job);
 	}
 
-#if 0
-	if (tab->priv->print_preview != NULL)
-	{
-		/* If we were printing while showing the print preview,
-		   see bug #352658 */
-		gtk_widget_destroy (tab->priv->print_preview);
-		g_return_if_fail (tab->priv->state == GEDIT_TAB_STATE_PRINTING);
-	}
-#endif
-
-	gedit_tab_set_state (tab, GEDIT_TAB_STATE_NORMAL);
+	close_printing (tab);
 
 	view = gedit_tab_get_view (tab);
 	gtk_widget_grab_focus (GTK_WIDGET (view));
-
-	if (tab->priv->print_job != NULL)
-	{
-		g_object_unref (tab->priv->print_job);
-		tab->priv->print_job = NULL;
-	}
 }
 
-#if 0
-static void
-print_preview_destroyed (GtkWidget *preview,
-			 GeditTab  *tab)
-{
-	tab->priv->print_preview = NULL;
-
-	if (tab->priv->state == GEDIT_TAB_STATE_SHOWING_PRINT_PREVIEW)
-	{
-		GeditView *view;
-
-		gedit_tab_set_state (tab, GEDIT_TAB_STATE_NORMAL);
-
-		view = gedit_tab_get_view (tab);
-		gtk_widget_grab_focus (GTK_WIDGET (view));
-	}
-	else
-	{
-		/* This should happen only when printing while showing the print
-		 * preview. In this case let us continue whithout changing
-		 * the state and show the document. See bug #352658 */
-		gtk_widget_show (tab->priv->view_scrolled_window);
-
-		g_return_if_fail (tab->priv->state == GEDIT_TAB_STATE_PRINTING);
-	}
-}
-#endif
-
 static void
 show_preview_cb (GeditPrintJob     *job,
 		 GeditPrintPreview *preview,
 		 GeditTab          *tab)
 {
-	/* g_return_if_fail (tab->priv->state == GEDIT_TAB_STATE_PRINT_PREVIEWING); */
 	g_return_if_fail (tab->priv->print_preview == NULL);
 
-	set_info_bar (tab, NULL, GTK_RESPONSE_NONE); /* destroy the info bar */
+	/* destroy the info bar */
+	set_info_bar (tab, NULL, GTK_RESPONSE_NONE);
 
 	tab->priv->print_preview = GTK_WIDGET (preview);
+	g_object_ref_sink (tab->priv->print_preview);
+
 	gtk_box_pack_end (GTK_BOX (tab),
 			  tab->priv->print_preview,
 			  TRUE,
 			  TRUE,
 			  0);
+
 	gtk_widget_show (tab->priv->print_preview);
 	gtk_widget_grab_focus (tab->priv->print_preview);
 
-/* when the preview gets destroyed we get "done" signal
-	g_signal_connect (tab->priv->print_preview,
-			  "destroy",
-			  G_CALLBACK (print_preview_destroyed),
-			  tab);
-*/
 	gedit_tab_set_state (tab, GEDIT_TAB_STATE_SHOWING_PRINT_PREVIEW);
 }
 
-#if 0
-
-static void
-set_print_preview (GeditTab  *tab,
-		   GtkWidget *print_preview)
-{
-	if (tab->priv->print_preview == print_preview)
-		return;
-
-	if (tab->priv->print_preview != NULL)
-		gtk_widget_destroy (tab->priv->print_preview);
-
-	tab->priv->print_preview = print_preview;
-
-	gtk_box_pack_end (GTK_BOX (tab),
-			  tab->priv->print_preview,
-			  TRUE,
-			  TRUE,
-			  0);
-
-	gtk_widget_grab_focus (tab->priv->print_preview);
-
-	g_signal_connect (tab->priv->print_preview,
-			  "destroy",
-			  G_CALLBACK (print_preview_destroyed),
-			  tab);
-}
-
-static void
-preview_finished_cb (GtkSourcePrintJob *pjob, GeditTab *tab)
-{
-	GnomePrintJob *gjob;
-	GtkWidget *preview = NULL;
-
-	g_return_if_fail (GEDIT_IS_PROGRESS_INFO_BAR (tab->priv->info_bar));
-	set_info_bar (tab, NULL, GTK_RESPONSE_NONE); /* destroy the info bar */
-
-	gjob = gtk_source_print_job_get_print_job (pjob);
-
-	preview = gedit_print_job_preview_new (gjob);
- 	g_object_unref (gjob);
-
-	set_print_preview (tab, preview);
-
-	gtk_widget_show (preview);
-	g_object_unref (pjob);
-
-	gedit_tab_set_state (tab, GEDIT_TAB_STATE_SHOWING_PRINT_PREVIEW);
-}
-
-
-#endif
-
 static void
 print_cancelled (GtkWidget *bar,
 		 gint       response_id,
 		 GeditTab  *tab)
 {
-	g_return_if_fail (GEDIT_IS_PROGRESS_INFO_BAR (tab->priv->info_bar));
+	gedit_debug (DEBUG_TAB);
 
-	gedit_print_job_cancel (tab->priv->print_job);
-
-	g_debug ("print_cancelled");
+	if (tab->priv->print_job != NULL)
+	{
+		gedit_print_job_cancel (tab->priv->print_job);
+	}
 }
 
 static void
@@ -2884,9 +2785,8 @@ show_printing_info_bar (GeditTab *tab)
 	set_info_bar (tab, bar, GTK_RESPONSE_NONE);
 }
 
-static void
-gedit_tab_print_or_print_preview (GeditTab                *tab,
-				  GtkPrintOperationAction  print_action)
+void
+_gedit_tab_print (GeditTab *tab)
 {
 	GeditView *view;
 	GtkPageSetup *setup;
@@ -2894,17 +2794,29 @@ gedit_tab_print_or_print_preview (GeditTab                *tab,
 	GtkPrintOperationResult res;
 	GError *error = NULL;
 
+	g_return_if_fail (GEDIT_IS_TAB (tab));
+
+	/* FIXME: currently we can have just one printoperation going on at a
+	 * given time, so before starting the print we close the preview.
+	 * Would be nice to handle it properly though.
+	 */
+	if (tab->priv->state == GEDIT_TAB_STATE_SHOWING_PRINT_PREVIEW)
+	{
+		close_printing (tab);
+	}
+
 	g_return_if_fail (tab->priv->print_job == NULL);
 	g_return_if_fail (tab->priv->state == GEDIT_TAB_STATE_NORMAL);
 
 	view = gedit_tab_get_view (tab);
 
 	tab->priv->print_job = gedit_print_job_new (view);
-	g_object_add_weak_pointer (G_OBJECT (tab->priv->print_job),
-				   (gpointer *) &tab->priv->print_job);
 
 	show_printing_info_bar (tab);
 
+	/* hide until we start printing */
+	gtk_widget_hide (tab->priv->info_bar);
+
 	g_signal_connect_object (tab->priv->print_job,
 				 "printing",
 				 G_CALLBACK (printing_cb),
@@ -2923,22 +2835,13 @@ gedit_tab_print_or_print_preview (GeditTab                *tab,
 				 tab,
 				 0);
 
-	if (print_action == GTK_PRINT_OPERATION_ACTION_PREVIEW)
-	{
-		gedit_tab_set_state (tab, GEDIT_TAB_STATE_PRINT_PREVIEWING);
-	}
-	else
-	{
-		/* hide until we start printing */
-		gtk_widget_hide (tab->priv->info_bar);
-		gedit_tab_set_state (tab, GEDIT_TAB_STATE_PRINTING);
-	}
+	gedit_tab_set_state (tab, GEDIT_TAB_STATE_PRINTING);
 
 	setup = get_page_setup (tab);
 	settings = get_print_settings (tab);
 
 	res = gedit_print_job_print (tab->priv->print_job,
-				     print_action,
+				     GTK_PRINT_OPERATION_ACTION_PRINT_DIALOG,
 				     setup,
 				     settings,
 				     GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (tab))),
@@ -2948,41 +2851,16 @@ gedit_tab_print_or_print_preview (GeditTab                *tab,
 	if (res == GTK_PRINT_OPERATION_RESULT_ERROR)
 	{
 		/* FIXME: go in error state */
-		gedit_tab_set_state (tab, GEDIT_TAB_STATE_NORMAL);
 		g_warning ("Async print preview failed (%s)", error->message);
-		g_object_unref (tab->priv->print_job);
 		g_error_free (error);
+
+		close_printing (tab);
 	}
 
+	g_object_unref (setup);
 	g_object_unref (settings);
 }
 
-void
-_gedit_tab_print (GeditTab *tab)
-{
-	g_return_if_fail (GEDIT_IS_TAB (tab));
-
-	/* FIXME: currently we can have just one printoperation going on
-	 * at a given time, so before starting the print we close the preview.
-	 * Would be nice to handle it properly though */
-	if (tab->priv->state == GEDIT_TAB_STATE_SHOWING_PRINT_PREVIEW)
-	{
-		gtk_widget_destroy (tab->priv->print_preview);
-	}
-
-	gedit_tab_print_or_print_preview (tab,
-					  GTK_PRINT_OPERATION_ACTION_PRINT_DIALOG);
-}
-
-void
-_gedit_tab_print_preview (GeditTab *tab)
-{
-	g_return_if_fail (GEDIT_IS_TAB (tab));
-
-	gedit_tab_print_or_print_preview (tab,
-					  GTK_PRINT_OPERATION_ACTION_PREVIEW);
-}
-
 void
 _gedit_tab_mark_for_closing (GeditTab *tab)
 {
